@@ -5,8 +5,6 @@ import warnings
 
 import attr
 
-__all__ = ["MultiError"]
-
 # python traceback.TracebackException < 3.6.4 does not support unhashable exceptions
 # see https://github.com/python/cpython/pull/4014 for details
 if sys.version_info < (3, 6, 4):
@@ -116,6 +114,9 @@ def _filter_impl(handler, root_exc):
     preserved = set()
     new_root_exc = filter_tree(root_exc, preserved)
     push_tb_down(None, root_exc, preserved)
+    # Delete the local functions to avoid a reference cycle (see
+    # test_simple_cancel_scope_usage_doesnt_create_cyclic_garbage)
+    del filter_tree, push_tb_down
     return new_root_exc
 
 
@@ -134,6 +135,7 @@ class MultiErrorCatcher:
     def __exit__(self, etype, exc, tb):
         if exc is not None:
             filtered_exc = MultiError.filter(self._handler, exc)
+
             if filtered_exc is exc:
                 # Let the interpreter re-raise it
                 return False
@@ -175,6 +177,7 @@ class MultiError(BaseException):
           :exc:`BaseException`.
 
     """
+
     def __init__(self, exceptions):
         # Avoid recursion when exceptions[0] returned by __new__() happens
         # to be a MultiError and subsequently __init__() is called.
@@ -188,9 +191,7 @@ class MultiError(BaseException):
         exceptions = list(exceptions)
         for exc in exceptions:
             if not isinstance(exc, BaseException):
-                raise TypeError(
-                    "Expected an exception object, not {!r}".format(exc)
-                )
+                raise TypeError("Expected an exception object, not {!r}".format(exc))
         if len(exceptions) == 1:
             # If this lone object happens to itself be a MultiError, then
             # Python will implicitly call our __init__ on it again.  See
@@ -282,16 +283,20 @@ if have_tproxy:
             # no missing test we could add, and no value in coverage nagging
             # us about adding one.
             if operation.opname in [
-                "__getattribute__", "__getattr__"
+                "__getattribute__",
+                "__getattr__",
             ]:  # pragma: no cover
                 if operation.args[0] == "tb_next":
                     return tb_next
             return operation.delegate()
 
         return tputil.make_proxy(controller, type(base_tb), base_tb)
+
+
 else:
     # ctypes it is
     import ctypes
+
     # How to handle refcounting? I don't want to use ctypes.py_object because
     # I don't understand or trust it, and I don't want to use
     # ctypes.pythonapi.Py_{Inc,Dec}Ref because we might clash with user code
@@ -376,10 +381,13 @@ def traceback_exception_init(
     limit=None,
     lookup_lines=True,
     capture_locals=False,
-    _seen=None
+    compact=False,
+    _seen=None,
 ):
-    if _seen is None:
-        _seen = set()
+    if sys.version_info >= (3, 10):
+        kwargs = {"compact": compact}
+    else:
+        kwargs = {}
 
     # Capture the original exception and its cause and context as TracebackExceptions
     traceback_exception_original_init(
@@ -390,9 +398,15 @@ def traceback_exception_init(
         limit=limit,
         lookup_lines=lookup_lines,
         capture_locals=capture_locals,
-        _seen=_seen
+        _seen=_seen,
+        **kwargs,
     )
 
+    seen_was_none = _seen is None
+
+    if _seen is None:
+        _seen = set()
+
     # Capture each of the exceptions in the MultiError along with each of their causes and contexts
     if isinstance(exc_value, MultiError):
         embedded = []
@@ -406,7 +420,7 @@ def traceback_exception_init(
                         capture_locals=capture_locals,
                         # copy the set of _seen exceptions so that duplicates
                         # shared between sub-exceptions are not omitted
-                        _seen=set(_seen)
+                        _seen=None if seen_was_none else set(_seen),
                     )
                 )
         self.embedded = embedded
@@ -414,7 +428,7 @@ def traceback_exception_init(
         self.embedded = []
 
 
-traceback.TracebackException.__init__ = traceback_exception_init
+traceback.TracebackException.__init__ = traceback_exception_init  # type: ignore
 traceback_exception_original_format = traceback.TracebackException.format
 
 
@@ -423,12 +437,10 @@ def traceback_exception_format(self, *, chain=True):
 
     for i, exc in enumerate(self.embedded):
         yield "\nDetails of embedded exception {}:\n\n".format(i + 1)
-        yield from (
-            textwrap.indent(line, " " * 2) for line in exc.format(chain=chain)
-        )
+        yield from (textwrap.indent(line, " " * 2) for line in exc.format(chain=chain))
 
 
-traceback.TracebackException.format = traceback_exception_format
+traceback.TracebackException.format = traceback_exception_format  # type: ignore
 
 
 def trio_excepthook(etype, value, tb):
@@ -436,10 +448,11 @@ def trio_excepthook(etype, value, tb):
         sys.stderr.write(chunk)
 
 
-IPython_handler_installed = False
-warning_given = False
+monkeypatched_or_warned = False
+
 if "IPython" in sys.modules:
     import IPython
+
     ip = IPython.get_ipython()
     if ip is not None:
         if ip.custom_exceptions != ():
@@ -448,9 +461,9 @@ if "IPython" in sys.modules:
                 "handler installed. I'll skip installing Trio's custom "
                 "handler, but this means MultiErrors will not show full "
                 "tracebacks.",
-                category=RuntimeWarning
+                category=RuntimeWarning,
             )
-            warning_given = True
+            monkeypatched_or_warned = True
         else:
 
             def trio_show_traceback(self, etype, value, tb, tb_offset=None):
@@ -459,15 +472,45 @@ if "IPython" in sys.modules:
                 trio_excepthook(etype, value, tb)
 
             ip.set_custom_exc((MultiError,), trio_show_traceback)
-            IPython_handler_installed = True
+            monkeypatched_or_warned = True
 
 if sys.excepthook is sys.__excepthook__:
     sys.excepthook = trio_excepthook
-else:
-    if not IPython_handler_installed and not warning_given:
-        warnings.warn(
-            "You seem to already have a custom sys.excepthook handler "
-            "installed. I'll skip installing Trio's custom handler, but this "
-            "means MultiErrors will not show full tracebacks.",
-            category=RuntimeWarning
-        )
+    monkeypatched_or_warned = True
+
+# Ubuntu's system Python has a sitecustomize.py file that import
+# apport_python_hook and replaces sys.excepthook.
+#
+# The custom hook captures the error for crash reporting, and then calls
+# sys.__excepthook__ to actually print the error.
+#
+# We don't mind it capturing the error for crash reporting, but we want to
+# take over printing the error. So we monkeypatch the apport_python_hook
+# module so that instead of calling sys.__excepthook__, it calls our custom
+# hook.
+#
+# More details: https://github.com/python-trio/trio/issues/1065
+if getattr(sys.excepthook, "__name__", None) == "apport_excepthook":
+    import apport_python_hook
+
+    assert sys.excepthook is apport_python_hook.apport_excepthook
+
+    # Give it a descriptive name as a hint for anyone who's stuck trying to
+    # debug this mess later.
+    class TrioFakeSysModuleForApport:
+        pass
+
+    fake_sys = TrioFakeSysModuleForApport()
+    fake_sys.__dict__.update(sys.__dict__)
+    fake_sys.__excepthook__ = trio_excepthook  # type: ignore
+    apport_python_hook.sys = fake_sys
+
+    monkeypatched_or_warned = True
+
+if not monkeypatched_or_warned:
+    warnings.warn(
+        "You seem to already have a custom sys.excepthook handler "
+        "installed. I'll skip installing Trio's custom handler, but this "
+        "means MultiErrors will not show full tracebacks.",
+        category=RuntimeWarning,
+    )
