@@ -75,19 +75,18 @@ import attr
 from collections import OrderedDict
 
 from .. import _core
-
-__all__ = ["ParkingLot"]
+from .._util import Final
 
 _counter = count()
 
 
-@attr.s(frozen=True)
+@attr.s(frozen=True, slots=True)
 class _ParkingLotStatistics:
     tasks_waiting = attr.ib()
 
 
-@attr.s(eq=False, hash=False)
-class ParkingLot:
+@attr.s(eq=False, hash=False, slots=True)
+class ParkingLot(metaclass=Final):
     """A fair wait queue with cancellation and requeueing.
 
     This class encapsulates the tricky parts of implementing a wait
@@ -105,15 +104,11 @@ class ParkingLot:
     _parked = attr.ib(factory=OrderedDict, init=False)
 
     def __len__(self):
-        """Returns the number of parked tasks.
-
-        """
+        """Returns the number of parked tasks."""
         return len(self._parked)
 
     def __bool__(self):
-        """True if there are parked tasks, False otherwise.
-
-        """
+        """True if there are parked tasks, False otherwise."""
         return bool(self._parked)
 
     # XX this currently returns None
@@ -159,9 +154,7 @@ class ParkingLot:
         return tasks
 
     def unpark_all(self):
-        """Unpark all parked tasks.
-
-        """
+        """Unpark all parked tasks."""
         return self.unpark(count=len(self))
 
     @_core.enable_ki_protection
@@ -177,8 +170,8 @@ class ParkingLot:
                print("woken")
 
            async def main():
-               lot1 = trio.hazmat.ParkingLot()
-               lot2 = trio.hazmat.ParkingLot()
+               lot1 = trio.lowlevel.ParkingLot()
+               lot2 = trio.lowlevel.ParkingLot()
                async with trio.open_nursery() as nursery:
                    nursery.start_soon(parker, lot1)
                    await trio.testing.wait_all_tasks_blocked()
