@@ -8,12 +8,6 @@ from .._highlevel_generic import aclose_forcefully
 from .._abc import SendStream, ReceiveStream, Stream, HalfCloseableStream
 from ._checkpoints import assert_checkpoints
 
-__all__ = [
-    "check_one_way_stream",
-    "check_two_way_stream",
-    "check_half_closeable_stream",
-]
-
 
 class _ForceCloseBoth:
     def __init__(self, both):
@@ -67,9 +61,9 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             with assert_checkpoints():
                 assert await s.send_all(data) is None
 
-        async def do_receive_some(max_bytes):
+        async def do_receive_some(*args):
             with assert_checkpoints():
-                return await r.receive_some(1)
+                return await r.receive_some(*args)
 
         async def checked_receive_1(expected):
             assert await do_receive_some(1) == expected
@@ -111,6 +105,13 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             await r.receive_some(0)
         with _assert_raises(TypeError):
             await r.receive_some(1.5)
+        # it can also be missing or None
+        async with _core.open_nursery() as nursery:
+            nursery.start_soon(do_send_all, b"x")
+            assert await do_receive_some() == b"x"
+        async with _core.open_nursery() as nursery:
+            nursery.start_soon(do_send_all, b"x")
+            assert await do_receive_some(None) == b"x"
 
         with _assert_raises(_core.BusyResourceError):
             async with _core.open_nursery() as nursery:
@@ -129,8 +130,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
 
         async with _core.open_nursery() as nursery:
             nursery.start_soon(
-                simple_check_wait_send_all_might_not_block,
-                nursery.cancel_scope
+                simple_check_wait_send_all_might_not_block, nursery.cancel_scope
             )
             nursery.start_soon(do_receive_some, 1)
 
@@ -164,6 +164,10 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
         with _assert_raises(_core.ClosedResourceError):
             await do_send_all(b"x" * 100)
 
+        # even if it's an empty send
+        with _assert_raises(_core.ClosedResourceError):
+            await do_send_all(b"")
+
         # ditto for wait_send_all_might_not_block
         with _assert_raises(_core.ClosedResourceError):
             with assert_checkpoints():
@@ -219,11 +223,11 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
 
     # cancelled aclose still closes
     async with _ForceCloseBoth(await stream_maker()) as (s, r):
-        with _core.open_cancel_scope() as scope:
+        with _core.CancelScope() as scope:
             scope.cancel()
             await r.aclose()
 
-        with _core.open_cancel_scope() as scope:
+        with _core.CancelScope() as scope:
             scope.cancel()
             await s.aclose()
 
@@ -248,7 +252,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             with _assert_raises(_core.Cancelled):
                 await afn(*args)
 
-        with _core.open_cancel_scope() as scope:
+        with _core.CancelScope() as scope:
             scope.cancel()
             async with _core.open_nursery() as nursery:
                 nursery.start_soon(expect_cancelled, do_send_all, b"x")
@@ -325,14 +329,15 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
                     nursery.start_soon(s.send_all, b"123")
                     nursery.start_soon(s.send_all, b"123")
 
-        # closing the receiver causes wait_send_all_might_not_block to return
+        # closing the receiver causes wait_send_all_might_not_block to return,
+        # with or without an exception
         async with _ForceCloseBoth(await clogged_stream_maker()) as (s, r):
 
             async def sender():
                 try:
                     with assert_checkpoints():
                         await s.wait_send_all_might_not_block()
-                except _core.BrokenResourceError:
+                except _core.BrokenResourceError:  # pragma: no cover
                     pass
 
             async def receiver():
@@ -349,7 +354,7 @@ async def check_one_way_stream(stream_maker, clogged_stream_maker):
             try:
                 with assert_checkpoints():
                     await s.wait_send_all_might_not_block()
-            except _core.BrokenResourceError:
+            except _core.BrokenResourceError:  # pragma: no cover
                 pass
 
         # Check that if a task is blocked in a send-side method, then closing
@@ -393,19 +398,18 @@ async def check_two_way_stream(stream_maker, clogged_stream_maker):
 
         async def flipped_clogged_stream_maker():
             return reversed(await clogged_stream_maker())
+
     else:
         flipped_clogged_stream_maker = None
-    await check_one_way_stream(
-        flipped_stream_maker, flipped_clogged_stream_maker
-    )
+    await check_one_way_stream(flipped_stream_maker, flipped_clogged_stream_maker)
 
     async with _ForceCloseBoth(await stream_maker()) as (s1, s2):
         assert isinstance(s1, Stream)
         assert isinstance(s2, Stream)
 
         # Duplex can be a bit tricky, might as well check it as well
-        DUPLEX_TEST_SIZE = 2**20
-        CHUNK_SIZE_MAX = 2**14
+        DUPLEX_TEST_SIZE = 2 ** 20
+        CHUNK_SIZE_MAX = 2 ** 14
 
         r = random.Random(0)
         i = r.getrandbits(8 * DUPLEX_TEST_SIZE)
