@@ -1,11 +1,14 @@
+import errno
+
 import pytest
+import attr
 
 import os
 import socket as stdlib_socket
 import inspect
 import tempfile
-
-from .._core.tests.tutil import need_ipv6
+import sys as _sys
+from .._core.tests.tutil import creates_ipv6, binds_ipv6
 from .. import _core
 from .. import _socket as _tsocket
 from .. import socket as tsocket
@@ -43,9 +46,7 @@ class MonkeypatchedGAI:
         elif bound[-1] & stdlib_socket.AI_NUMERICHOST:
             return self._orig_getaddrinfo(*args, **kwargs)
         else:
-            raise RuntimeError(
-                "gai called with unexpected arguments {}".format(bound)
-            )
+            raise RuntimeError("gai called with unexpected arguments {}".format(bound))
 
 
 @pytest.fixture
@@ -100,37 +101,51 @@ def test_socket_has_some_reexports():
 async def test_getaddrinfo(monkeygai):
     def check(got, expected):
         # win32 returns 0 for the proto field
-        def without_proto(gai_tup):
-            return gai_tup[:2] + (0,) + gai_tup[3:]
+        # musl and glibc have inconsistent handling of the canonical name
+        # field (https://github.com/python-trio/trio/issues/1499)
+        # Neither field gets used much and there isn't much opportunity for us
+        # to mess them up, so we don't bother checking them here
+        def interesting_fields(gai_tup):
+            # (family, type, proto, canonname, sockaddr)
+            family, type, proto, canonname, sockaddr = gai_tup
+            return (family, type, sockaddr)
 
-        expected2 = [without_proto(gt) for gt in expected]
-        assert got == expected or got == expected2
+        def filtered(gai_list):
+            return [interesting_fields(gai_tup) for gai_tup in gai_list]
+
+        assert filtered(got) == filtered(expected)
 
     # Simple non-blocking non-error cases, ipv4 and ipv6:
     with assert_checkpoints():
-        res = await tsocket.getaddrinfo(
-            "127.0.0.1", "12345", type=tsocket.SOCK_STREAM
-        )
+        res = await tsocket.getaddrinfo("127.0.0.1", "12345", type=tsocket.SOCK_STREAM)
 
-    check(res, [
-        (tsocket.AF_INET,  # 127.0.0.1 is ipv4
-         tsocket.SOCK_STREAM,
-         tsocket.IPPROTO_TCP,
-         "",
-         ("127.0.0.1", 12345)),
-    ])  # yapf: disable
+    check(
+        res,
+        [
+            (
+                tsocket.AF_INET,  # 127.0.0.1 is ipv4
+                tsocket.SOCK_STREAM,
+                tsocket.IPPROTO_TCP,
+                "",
+                ("127.0.0.1", 12345),
+            ),
+        ],
+    )
 
     with assert_checkpoints():
-        res = await tsocket.getaddrinfo(
-            "::1", "12345", type=tsocket.SOCK_DGRAM
-        )
-    check(res, [
-        (tsocket.AF_INET6,
-         tsocket.SOCK_DGRAM,
-         tsocket.IPPROTO_UDP,
-         "",
-         ("::1", 12345, 0, 0)),
-    ])  # yapf: disable
+        res = await tsocket.getaddrinfo("::1", "12345", type=tsocket.SOCK_DGRAM)
+    check(
+        res,
+        [
+            (
+                tsocket.AF_INET6,
+                tsocket.SOCK_DGRAM,
+                tsocket.IPPROTO_UDP,
+                "",
+                ("::1", 12345, 0, 0),
+            ),
+        ],
+    )
 
     monkeygai.set("x", b"host", "port", family=0, type=0, proto=0, flags=0)
     with assert_checkpoints():
@@ -142,8 +157,10 @@ async def test_getaddrinfo(monkeygai):
     with assert_checkpoints():
         with pytest.raises(tsocket.gaierror) as excinfo:
             await tsocket.getaddrinfo("::1", "12345", type=-1)
-    # Linux, Windows
+    # Linux + glibc, Windows
     expected_errnos = {tsocket.EAI_SOCKTYPE}
+    # Linux + musl
+    expected_errnos.add(tsocket.EAI_SERVICE)
     # macOS
     if hasattr(tsocket, "EAI_BADHINTS"):
         expected_errnos.add(tsocket.EAI_BADHINTS)
@@ -209,9 +226,9 @@ async def test_from_stdlib_socket():
     class MySocket(stdlib_socket.socket):
         pass
 
-    mysock = MySocket()
-    with pytest.raises(TypeError):
-        tsocket.from_stdlib_socket(mysock)
+    with MySocket() as mysock:
+        with pytest.raises(TypeError):
+            tsocket.from_stdlib_socket(mysock)
 
 
 async def test_from_fd():
@@ -254,11 +271,41 @@ async def test_socket():
         assert isinstance(s, tsocket.SocketType)
         assert s.family == tsocket.AF_INET
 
+
+@creates_ipv6
+async def test_socket_v6():
     with tsocket.socket(tsocket.AF_INET6, tsocket.SOCK_DGRAM) as s:
         assert isinstance(s, tsocket.SocketType)
         assert s.family == tsocket.AF_INET6
 
 
+@pytest.mark.skipif(not _sys.platform == "linux", reason="linux only")
+async def test_sniff_sockopts():
+    from socket import AF_INET, AF_INET6, SOCK_DGRAM, SOCK_STREAM
+
+    # generate the combinations of families/types we're testing:
+    sockets = []
+    for family in [AF_INET, AF_INET6]:
+        for type in [SOCK_DGRAM, SOCK_STREAM]:
+            sockets.append(stdlib_socket.socket(family, type))
+    for socket in sockets:
+        # regular Trio socket constructor
+        tsocket_socket = tsocket.socket(fileno=socket.fileno())
+        # check family / type for correctness:
+        assert tsocket_socket.family == socket.family
+        assert tsocket_socket.type == socket.type
+        tsocket_socket.detach()
+
+        # fromfd constructor
+        tsocket_from_fd = tsocket.fromfd(socket.fileno(), AF_INET, SOCK_STREAM)
+        # check family / type for correctness:
+        assert tsocket_from_fd.family == socket.family
+        assert tsocket_from_fd.type == socket.type
+        tsocket_from_fd.close()
+
+        socket.close()
+
+
 ################################################################
 # _SocketType
 ################################################################
@@ -355,10 +402,11 @@ async def test_SocketType_shutdown():
 
 
 @pytest.mark.parametrize(
-    "address, socket_type", [
-        ('127.0.0.1', tsocket.AF_INET),
-        pytest.param('::1', tsocket.AF_INET6, marks=need_ipv6)
-    ]
+    "address, socket_type",
+    [
+        ("127.0.0.1", tsocket.AF_INET),
+        pytest.param("::1", tsocket.AF_INET6, marks=binds_ipv6),
+    ],
 )
 async def test_SocketType_simple_server(address, socket_type):
     # listen, bind, accept, connect, getpeername, getsockname
@@ -377,6 +425,17 @@ async def test_SocketType_simple_server(address, socket_type):
             assert await client.recv(1) == b"x"
 
 
+async def test_SocketType_is_readable():
+    a, b = tsocket.socketpair()
+    with a, b:
+        assert not a.is_readable()
+        await b.send(b"x")
+        await _core.wait_readable(a)
+        assert a.is_readable()
+        assert await a.recv(1) == b"x"
+        assert not a.is_readable()
+
+
 # On some macOS systems, getaddrinfo likes to return V4-mapped addresses even
 # when we *don't* pass AI_V4MAPPED.
 # https://github.com/python-trio/trio/issues/580
@@ -389,86 +448,130 @@ def gai_without_v4mapped_is_buggy():  # pragma: no cover
         return True
 
 
+@attr.s
+class Addresses:
+    bind_all = attr.ib()
+    localhost = attr.ib()
+    arbitrary = attr.ib()
+    broadcast = attr.ib()
+
+
 # Direct thorough tests of the implicit resolver helpers
-async def test_SocketType_resolve():
-    # For some reason the stdlib special-cases "" to pass NULL to getaddrinfo
-    # They also error out on None, but whatever, None is much more consistent,
-    # so we accept it too.
-    for null in [None, ""]:
-        sock4 = tsocket.socket(family=tsocket.AF_INET)
-        got = await sock4._resolve_local_address((null, 80))
-        assert got == ("0.0.0.0", 80)
-        got = await sock4._resolve_remote_address((null, 80))
-        assert got == ("127.0.0.1", 80)
+@pytest.mark.parametrize(
+    "socket_type, addrs",
+    [
+        (
+            tsocket.AF_INET,
+            Addresses(
+                bind_all="0.0.0.0",
+                localhost="127.0.0.1",
+                arbitrary="1.2.3.4",
+                broadcast="255.255.255.255",
+            ),
+        ),
+        pytest.param(
+            tsocket.AF_INET6,
+            Addresses(
+                bind_all="::",
+                localhost="::1",
+                arbitrary="1::2",
+                broadcast="::ffff:255.255.255.255",
+            ),
+            marks=creates_ipv6,
+        ),
+    ],
+)
+async def test_SocketType_resolve(socket_type, addrs):
+    v6 = socket_type == tsocket.AF_INET6
 
-        sock6 = tsocket.socket(family=tsocket.AF_INET6)
-        got = await sock6._resolve_local_address((null, 80))
-        assert got == ("::", 80, 0, 0)
+    def pad(addr):
+        if v6:
+            while len(addr) < 4:
+                addr += (0,)
+        return addr
 
-        got = await sock6._resolve_remote_address((null, 80))
-        assert got == ("::1", 80, 0, 0)
+    def assert_eq(actual, expected):
+        assert pad(expected) == pad(actual)
 
-    # AI_PASSIVE only affects the wildcard address, so for everything else
-    # _resolve_local_address and _resolve_remote_address should work the same:
-    for res in ["_resolve_local_address", "_resolve_remote_address"]:
+    with tsocket.socket(family=socket_type) as sock:
+        # For some reason the stdlib special-cases "" to pass NULL to
+        # getaddrinfo They also error out on None, but whatever, None is much
+        # more consistent, so we accept it too.
+        for null in [None, ""]:
+            got = await sock._resolve_local_address_nocp((null, 80))
+            assert_eq(got, (addrs.bind_all, 80))
+            got = await sock._resolve_remote_address_nocp((null, 80))
+            assert_eq(got, (addrs.localhost, 80))
 
-        async def s4res(*args):
-            return await getattr(sock4, res)(*args)
+        # AI_PASSIVE only affects the wildcard address, so for everything else
+        # _resolve_local_address_nocp and _resolve_remote_address_nocp should
+        # work the same:
+        for resolver in ["_resolve_local_address_nocp", "_resolve_remote_address_nocp"]:
 
-        async def s6res(*args):
-            return await getattr(sock6, res)(*args)
+            async def res(*args):
+                return await getattr(sock, resolver)(*args)
 
-        assert await s4res(("1.2.3.4", "http")) == ("1.2.3.4", 80)
-        assert await s6res(("1::2", "http")) == ("1::2", 80, 0, 0)
+            assert_eq(await res((addrs.arbitrary, "http")), (addrs.arbitrary, 80))
+            if v6:
+                # Check handling of different length ipv6 address tuples
+                assert_eq(await res(("1::2", 80)), ("1::2", 80, 0, 0))
+                assert_eq(await res(("1::2", 80, 0)), ("1::2", 80, 0, 0))
+                assert_eq(await res(("1::2", 80, 0, 0)), ("1::2", 80, 0, 0))
+                # Non-zero flowinfo/scopeid get passed through
+                assert_eq(await res(("1::2", 80, 1)), ("1::2", 80, 1, 0))
+                assert_eq(await res(("1::2", 80, 1, 2)), ("1::2", 80, 1, 2))
 
-        assert await s6res(("1::2", 80, 1)) == ("1::2", 80, 1, 0)
-        assert await s6res(("1::2", 80, 1, 2)) == ("1::2", 80, 1, 2)
+                # And again with a string port, as a trick to avoid the
+                # already-resolved address fastpath and make sure we call
+                # getaddrinfo
+                assert_eq(await res(("1::2", "80")), ("1::2", 80, 0, 0))
+                assert_eq(await res(("1::2", "80", 0)), ("1::2", 80, 0, 0))
+                assert_eq(await res(("1::2", "80", 0, 0)), ("1::2", 80, 0, 0))
+                assert_eq(await res(("1::2", "80", 1)), ("1::2", 80, 1, 0))
+                assert_eq(await res(("1::2", "80", 1, 2)), ("1::2", 80, 1, 2))
 
-        # V4 mapped addresses resolved if V6ONLY is False
-        sock6.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, False)
-        assert await s6res(("1.2.3.4", "http")) == ("::ffff:1.2.3.4", 80, 0, 0)
+                # V4 mapped addresses resolved if V6ONLY is False
+                sock.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, False)
+                assert_eq(await res(("1.2.3.4", "http")), ("::ffff:1.2.3.4", 80))
 
-        # Check the <broadcast> special case, because why not
-        await s4res(("<broadcast>", 123)) == ("255.255.255.255", 123)
-        await s6res(("<broadcast>", 123)) == ("::ffff:255.255.255.255", 123)
+            # Check the <broadcast> special case, because why not
+            assert_eq(await res(("<broadcast>", 123)), (addrs.broadcast, 123))
 
-        # But not if it's true (at least on systems where getaddrinfo works
-        # correctly)
-        if not gai_without_v4mapped_is_buggy():  # pragma: no branch
-            sock6.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, True)
-            with pytest.raises(tsocket.gaierror) as excinfo:
-                await s6res(("1.2.3.4", 80))
-            # Windows, macOS
-            expected_errnos = {tsocket.EAI_NONAME}
-            # Linux
-            if hasattr(tsocket, "EAI_ADDRFAMILY"):
-                expected_errnos.add(tsocket.EAI_ADDRFAMILY)
-            assert excinfo.value.errno in expected_errnos
+            # But not if it's true (at least on systems where getaddrinfo works
+            # correctly)
+            if v6 and not gai_without_v4mapped_is_buggy():
+                sock.setsockopt(tsocket.IPPROTO_IPV6, tsocket.IPV6_V6ONLY, True)
+                with pytest.raises(tsocket.gaierror) as excinfo:
+                    await res(("1.2.3.4", 80))
+                # Windows, macOS
+                expected_errnos = {tsocket.EAI_NONAME}
+                # Linux
+                if hasattr(tsocket, "EAI_ADDRFAMILY"):
+                    expected_errnos.add(tsocket.EAI_ADDRFAMILY)
+                assert excinfo.value.errno in expected_errnos
 
-        # A family where we know nothing about the addresses, so should just
-        # pass them through. This should work on Linux, which is enough to
-        # smoke test the basic functionality...
-        try:
-            netlink_sock = tsocket.socket(
-                family=tsocket.AF_NETLINK, type=tsocket.SOCK_DGRAM
-            )
-        except (AttributeError, OSError):
-            pass
-        else:
-            assert await getattr(netlink_sock, res)("asdf") == "asdf"
+            # A family where we know nothing about the addresses, so should just
+            # pass them through. This should work on Linux, which is enough to
+            # smoke test the basic functionality...
+            try:
+                netlink_sock = tsocket.socket(
+                    family=tsocket.AF_NETLINK, type=tsocket.SOCK_DGRAM
+                )
+            except (AttributeError, OSError):
+                pass
+            else:
+                assert await getattr(netlink_sock, resolver)("asdf") == "asdf"
+                netlink_sock.close()
 
-        with pytest.raises(ValueError):
-            await s4res("1.2.3.4")
-        with pytest.raises(ValueError):
-            await s4res(("1.2.3.4",))
-        with pytest.raises(ValueError):
-            await s4res(("1.2.3.4", 80, 0, 0))
-        with pytest.raises(ValueError):
-            await s6res("1.2.3.4")
-        with pytest.raises(ValueError):
-            await s6res(("1.2.3.4",))
-        with pytest.raises(ValueError):
-            await s6res(("1.2.3.4", 80, 0, 0, 0))
+            with pytest.raises(ValueError):
+                await res("1.2.3.4")
+            with pytest.raises(ValueError):
+                await res(("1.2.3.4",))
+            with pytest.raises(ValueError):
+                if v6:
+                    await res(("1.2.3.4", 80, 0, 0, 0))
+                else:
+                    await res(("1.2.3.4", 80, 0, 0))
 
 
 async def test_SocketType_unresolved_names():
@@ -498,12 +601,12 @@ async def test_SocketType_non_blocking_paths():
 
         # cancel before even calling
         b.send(b"1")
-        with _core.open_cancel_scope() as cscope:
+        with _core.CancelScope() as cscope:
             cscope.cancel()
             with assert_checkpoints():
                 with pytest.raises(_core.Cancelled):
                     await ta.recv(10)
-        # immedate success (also checks that the previous attempt didn't
+        # immediate success (also checks that the previous attempt didn't
         # actually read anything)
         with assert_checkpoints():
             await ta.recv(10) == b"1"
@@ -567,21 +670,19 @@ async def test_SocketType_non_blocking_paths():
 # This tests the complicated paths through connect
 async def test_SocketType_connect_paths():
     with tsocket.socket() as sock:
-        with assert_checkpoints():
-            with pytest.raises(ValueError):
-                # Should be a tuple
-                await sock.connect("localhost")
+        with pytest.raises(ValueError):
+            # Should be a tuple
+            await sock.connect("localhost")
 
     # cancelled before we start
     with tsocket.socket() as sock:
-        with assert_checkpoints():
-            with _core.open_cancel_scope() as cancel_scope:
-                cancel_scope.cancel()
-                with pytest.raises(_core.Cancelled):
-                    await sock.connect(("127.0.0.1", 80))
+        with _core.CancelScope() as cancel_scope:
+            cancel_scope.cancel()
+            with pytest.raises(_core.Cancelled):
+                await sock.connect(("127.0.0.1", 80))
 
     # Cancelled in between the connect() call and the connect completing
-    with _core.open_cancel_scope() as cancel_scope:
+    with _core.CancelScope() as cancel_scope:
         with tsocket.socket() as sock, tsocket.socket() as listener:
             await listener.bind(("127.0.0.1", 0))
             listener.listen()
@@ -609,31 +710,30 @@ async def test_SocketType_connect_paths():
 
     # Failed connect (hopefully after raising BlockingIOError)
     with tsocket.socket() as sock:
-        with assert_checkpoints():
-            with pytest.raises(OSError):
-                # TCP port 2 is not assigned. Pretty sure nothing will be
-                # listening there. (We used to bind a port and then *not* call
-                # listen() to ensure nothing was listening there, but it turns
-                # out on macOS if you do this it takes 30 seconds for the
-                # connect to fail. Really. Also if you use a non-routable
-                # address. This way fails instantly though. As long as nothing
-                # is listening on port 2.)
-                await sock.connect(("127.0.0.1", 2))
+        with pytest.raises(OSError):
+            # TCP port 2 is not assigned. Pretty sure nothing will be
+            # listening there. (We used to bind a port and then *not* call
+            # listen() to ensure nothing was listening there, but it turns
+            # out on macOS if you do this it takes 30 seconds for the
+            # connect to fail. Really. Also if you use a non-routable
+            # address. This way fails instantly though. As long as nothing
+            # is listening on port 2.)
+            await sock.connect(("127.0.0.1", 2))
 
 
 async def test_resolve_remote_address_exception_closes_socket():
     # Here we are testing issue 247, any cancellation will leave the socket closed
-    with _core.open_cancel_scope() as cancel_scope:
+    with _core.CancelScope() as cancel_scope:
         with tsocket.socket() as sock:
 
-            async def _resolve_remote_address(self, *args, **kwargs):
+            async def _resolve_remote_address_nocp(self, *args, **kwargs):
                 cancel_scope.cancel()
                 await _core.checkpoint()
 
-            sock._resolve_remote_address = _resolve_remote_address
+            sock._resolve_remote_address_nocp = _resolve_remote_address_nocp
             with assert_checkpoints():
                 with pytest.raises(_core.Cancelled):
-                    await sock.connect('')
+                    await sock.connect("")
             assert sock.fileno() == -1
 
 
@@ -779,9 +879,11 @@ async def test_custom_hostname_resolver(monkeygai):
         (0, 0, tsocket.IPPROTO_TCP, 0),
         (0, 0, 0, tsocket.AI_CANONNAME),
     ]:
-        assert (
-            await tsocket.getaddrinfo("localhost", "foo", *vals) ==
-            ("custom_gai", b"localhost", "foo", *vals)
+        assert await tsocket.getaddrinfo("localhost", "foo", *vals) == (
+            "custom_gai",
+            b"localhost",
+            "foo",
+            *vals,
         )
 
     # IDNA encoding is handled before calling the special object
@@ -789,12 +891,12 @@ async def test_custom_hostname_resolver(monkeygai):
     expected = ("custom_gai", b"xn--f-1gaa", "foo", 0, 0, 0, 0)
     assert got == expected
 
-    assert (await tsocket.getnameinfo("a", 0) == ("custom_gni", "a", 0))
+    assert await tsocket.getnameinfo("a", 0) == ("custom_gni", "a", 0)
 
     # We can set it back to None
     assert tsocket.set_custom_hostname_resolver(None) is cr
 
-    # And now trio switches back to calling socket.getaddrinfo (specifically
+    # And now Trio switches back to calling socket.getaddrinfo (specifically
     # our monkeypatched version of socket.getaddrinfo)
     monkeygai.set("x", b"host", "port", family=0, type=0, proto=0, flags=0)
     assert await tsocket.getaddrinfo("host", "port") == "x"
@@ -832,9 +934,7 @@ async def test_SocketType_is_abstract():
         tsocket.SocketType()
 
 
-@pytest.mark.skipif(
-    not hasattr(tsocket, "AF_UNIX"), reason="no unix domain sockets"
-)
+@pytest.mark.skipif(not hasattr(tsocket, "AF_UNIX"), reason="no unix domain sockets")
 async def test_unix_domain_socket():
     # Bind has a special branch to use a thread, since it has to do filesystem
     # traversal. Maybe connect should too? Not sure.
@@ -846,8 +946,9 @@ async def test_unix_domain_socket():
             with tsocket.socket(family=tsocket.AF_UNIX) as csock:
                 await csock.connect(path)
                 ssock, _ = await lsock.accept()
-                await csock.send(b"x")
-                assert await ssock.recv(1) == b"x"
+                with ssock:
+                    await csock.send(b"x")
+                    assert await ssock.recv(1) == b"x"
 
     # Can't use tmpdir fixture, because we can exceed the maximum AF_UNIX path
     # length on macOS.
@@ -891,3 +992,24 @@ async def test_interrupted_by_close():
             nursery.start_soon(receiver)
             await wait_all_tasks_blocked()
             a.close()
+
+
+async def test_many_sockets():
+    total = 5000  # Must be more than MAX_AFD_GROUP_SIZE
+    sockets = []
+    for x in range(total // 2):
+        try:
+            a, b = stdlib_socket.socketpair()
+        except OSError as e:  # pragma: no cover
+            assert e.errno in (errno.EMFILE, errno.ENFILE)
+            break
+        sockets += [a, b]
+    async with _core.open_nursery() as nursery:
+        for s in sockets:
+            nursery.start_soon(_core.wait_readable, s)
+        await _core.wait_all_tasks_blocked()
+        nursery.cancel_scope.cancel()
+    for sock in sockets:
+        sock.close()
+    if x != total // 2 - 1:  # pragma: no cover
+        print(f"Unable to open more than {(x-1)*2} sockets.")
